class ValidationError(Exception):
    def __init__(self, message):
        super(ValidationError, self).__init__(message)


class OperationNotPermitted(Exception):
    def __init__(self, message):
        super(OperationNotPermitted, self).__init__(message)


class MalformedJsonRequest(Exception):
    def __init__(self, message):
        super(MalformedJsonRequest, self).__init__(message)


class RequestFailed(Exception):
    def __init__(self, message):
        super(RequestFailed, self).__init__(message)


class OperationAlreadyEnqueued(Exception):
    def __init__(self, message):
        super(OperationAlreadyEnqueued, self).__init__(message)



import json
from abc import ABCMeta
from collections import OrderedDict
from ArubaCloud.base import JsonInterfaceBase


class Creator(object):
    __metaclass__ = ABCMeta
    json_msg = OrderedDict()

    def get_raw(self):
        return self.json_msg

    def get_json(self):
        return json.dumps(self.json_msg)

    def commit(self, url, debug=False):
        from ArubaCloud.helper import Http
        url = '{}/{}'.format(url, 'SetAddServerScheduledOperation')
        headers = {'Content-Type': 'application/json', 'Content-Length': len(self.get_json())}
        response = Http.post(url=url, data=self.get_json(), headers=headers)
        parsed_response = json.loads(response.content)
        if debug is True:
            print(parsed_response)
        if parsed_response["Success"]:
            return True
        return False


class Scheduled(JsonInterfaceBase):
    def __init__(self, DC):
        super(Scheduled, self).__init__()
        self.wcf_baseurl = 'https://api.dc%s.computing.cloud.it/WsEndUser/v2.9/WsEndUser.svc/json' % (str(DC))

    @property
    def name(self):
        return self.name

    def get_scheduled_operations(self, start, end):
        
        get_shed_ops_request = {
            "GetScheduledOperations": {
                "EndDate": end,
                "StartDate": start
            }
        }
        scheme = self.gen_def_json_scheme('GetScheduledOperations', method_fields=get_shed_ops_request)
        json_obj = self.call_method_post('GetScheduledOperations', json_scheme=scheme)
        return json_obj

    def suspend_scheduled_operation(self, operationid):
        suspend_request = {
            "SetUpdateServerScheduledOperation": {
                "ScheduledOperationId": operationid
            }
        }
        scheme = self.gen_def_json_scheme('SetUpdateServerScheduledOperation', method_fields=suspend_request)
        json_obj = self.call_method_post('SetUpdateServerScheduledOperation', json_scheme=scheme)
        return True if json_obj['Success'] is True else False

    def delete_scheduled_operation(self, operationid):
        delete_request = {
            "SetRemoveServerScheduledOperation": {
                "ScheduledOperationId": operationid
            }
        }
        scheme = self.gen_def_json_scheme('SetRemoveServerScheduledOperation', method_fields=delete_request)
        json_obj = self.call_method_post('SetRemoveServerScheduledOperation', json_scheme=scheme)
        return True if json_obj['Success'] is True else False


class ScheduledCreator(Creator):
    def __init__(self, name, serverid, operationtype, start, end, DC, auth_obj):
        self.name = name
        self.serverid = serverid
        self.operationtype = ['CreateSnapshot', 'DeleteSnapshot', 'RestoreSnapShot',
                              'ShutdownVirtualMachine', 'StartVirtualMachine',
                              'StopVirtualMachine', 'UpdateVirtualMachine']
        for _optype_ in operationtype:
            if operationtype not in _optype_:
                raise Exception('error')
        self.start = start
        self.end = end
        self.auth = auth_obj
        self.wcf_baseurl = 'https://api.dc%s.computing.cloud.it/WsEndUser/v2.9/WsEndUser.svc/json' % (str(DC))
        self.json_msg = {
            'ApplicationId': 'SetAddServerScheduledOperation',
            'RequestId': 'SetAddServerScheduledOperation',
            'SessionId': '',
            'Password': auth_obj.password,
            'Username': self.auth.username,
            'SetAddServerScheduledOperation': {
                'ScheduledOperationTypes': self.operationtype,
                'ScheduleOperationLabel': self.name,
                'ServerID': self.serverid,
                'ScheduleStartDateTime': self.start,
                'ScheduleEndDateTime': self.end,
                'ScheduleFrequencyType': 'RunOnce',
                'ScheduledPlanStatus': 'Enabled',
                'ScheduledMontlyRecurrence': 'FirstDay'
            }
        }



import logging
import sys
from pprint import pprint

from ArubaCloud.base import Auth, JsonInterfaceBase
from ArubaCloud.base.logsystem import ArubaLog
from ArubaCloud.base.vm import VMList
from ArubaCloud.objects import Ip, IpList, Vlan
from ArubaCloud.objects.Templates import Template
from ArubaCloud.objects.VmTypes import Pro, Smart


class JsonInterface(JsonInterfaceBase):
    pass


class LoadBalancer(JsonInterfaceBase):
    def __init__(self):
        super(LoadBalancer, self).__init__()
        self._name = ''
        self.auth = Auth()

    @property
    def name(self):
        return self._name

    def get(self):
        scheme = self.gen_def_json_scheme('GetLoadBalancers')
        json_obj = self.call_method_post('GetLoadbalancers', json_scheme=scheme)
        print(json_obj)

    def login(self, username, password):
        self.auth.username = username
        self.auth.password = password


class CloudInterface(JsonInterface):
    templates = []
    vmlist = VMList()
    iplist = IpList()
    json_templates = None
    json_servers = None
    ip_resource = None
    hypervisors = {3: "LC", 4: "SMART", 2: "VW", 1: "HV"}

    def __init__(self, dc, debug_level=logging.INFO):
        super(CloudInterface, self).__init__()
        assert isinstance(dc, int), Exception('dc must be an integer and must be not null.')
        self.wcf_baseurl = 'https://api.dc%s.computing.cloud.it/WsEndUser/v2.9/WsEndUser.svc/json' % (str(dc))
        self.logger = ArubaLog(level=debug_level, log_to_file=False)
        self.logger.name = self.__class__
        self.auth = None

    def login(self, username, password, load=True):
        """
        Set the authentication data in the object, and if load is True
        (default is True) it also retrieve the ip list and the vm list
        in order to build the internal objects list.
        @param (str) username: username of the cloud
        @param (str) password: password of the cloud
        @param (bool) load: define if pre cache the objects.
        @return: None
        """
        self.auth = Auth(username, password)
        if load is True:
            self.get_ip()
            self.get_servers()

    def poweroff_server(self, server=None, server_id=None):
        """
        Poweroff a VM. If possible to pass the VM object or simply the ID
        of the VM that we want to turn on.
        Args:
            server: VM Object that represent the VM to power off,
            server_id: Int or Str representing the ID of the VM to power off.
        Returns:
            return True if json_obj['Success'] is 'True' else False
        """
        sid = server_id if server_id is not None else server.sid
        if sid is None:
            raise Exception('No Server Specified.')
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerPowerOff', dict(ServerId=sid))
        json_obj = self.call_method_post('SetEnqueueServerPowerOff', json_scheme=json_scheme)
        return True if json_obj['Success'] is 'True' else False

    def poweron_server(self, server=None, server_id=None):
        """
        Poweron a VM. If possible to pass the VM object or simply the ID
        of the VM that we want to turn on.
        Args:
            server: VM Object that represent the VM to power on,
            server_id: Int or Str representing the ID of the VM to power on.
        Returns:
            return True if json_obj['Success'] is 'True' else False
        """
        sid = server_id if server_id is not None else server.sid
        if sid is None:
            raise Exception('No Server Specified.')
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerStart', dict(ServerId=sid))
        json_obj = self.call_method_post('SetEnqueueServerStart', json_scheme=json_scheme)
        return True if json_obj['Success'] is 'True' else False

    def get_hypervisors(self):
        """
        Initialize the internal list containing each template available for each
        hypervisor.

        :return: [bool] True in case of success, otherwise False
        """
        json_scheme = self.gen_def_json_scheme('GetHypervisors')
        json_obj = self.call_method_post(method='GetHypervisors', json_scheme=json_scheme)
        self.json_templates = json_obj
        d = dict(json_obj)
        for elem in d['Value']:
            hv = self.hypervisors[elem['HypervisorType']]
            for inner_elem in elem['Templates']:
                o = Template(hv)
                o.template_id = inner_elem['Id']
                o.descr = inner_elem['Description']
                o.id_code = inner_elem['IdentificationCode']
                o.name = inner_elem['Name']
                o.enabled = inner_elem['Enabled']
                if hv != 'SMART':
                    for rb in inner_elem['ResourceBounds']:
                        resource_type = rb['ResourceType']
                        if resource_type == 1:
                            o.resource_bounds.max_cpu = rb['Max']
                        if resource_type == 2:
                            o.resource_bounds.max_memory = rb['Max']
                        if resource_type == 3:
                            o.resource_bounds.hdd0 = rb['Max']
                        if resource_type == 7:
                            o.resource_bounds.hdd1 = rb['Max']
                        if resource_type == 8:
                            o.resource_bounds.hdd2 = rb['Max']
                        if resource_type == 9:
                            o.resource_bounds.hdd3 = rb['Max']
                self.templates.append(o)
        return True if json_obj['Success'] is 'True' else False

    def get_servers(self):
        """
        Create the list of Server object inside the Datacenter objects.
        Build an internal list of VM Objects (pro or smart) as iterator.
        :return: bool
        """
        json_scheme = self.gen_def_json_scheme('GetServers')
        json_obj = self.call_method_post(method='GetServers', json_scheme=json_scheme)
        self.json_servers = json_obj
        # if this method is called I assume that i must re-read the data
        # so i reinitialize the vmlist
        self.vmlist = VMList()
        # getting all instanced IP in case the list is empty
        if len(self.iplist) <= 0:
            self.get_ip()
        for elem in dict(json_obj)["Value"]:
            if elem['HypervisorType'] is 4:
                s = Smart(interface=self, sid=elem['ServerId'])
            else:
                s = Pro(interface=self, sid=elem['ServerId'])
            s.vm_name = elem['Name']
            s.cpu_qty = elem['CPUQuantity']
            s.ram_qty = elem['RAMQuantity']
            s.status = elem['ServerStatus']
            s.datacenter_id = elem['DatacenterId']
            s.wcf_baseurl = self.wcf_baseurl
            s.auth = self.auth
            s.hd_qty = elem['HDQuantity']
            s.hd_total_size = elem['HDTotalSize']
            if elem['HypervisorType'] is 4:
                ssd = self.get_server_detail(elem['ServerId'])
                try:
                    s.ip_addr = str(ssd['EasyCloudIPAddress']['Value'])
                except TypeError:
                    s.ip_addr = 'Not retrieved.'
            else:
                s.ip_addr = []
                for ip in self.iplist:
                    if ip.serverid == s.sid:
                        s.ip_addr.append(ip)
            self.vmlist.append(s)
        return True if json_obj['Success'] is True else False

    def find_template(self, name=None, hv=None):
        """
        Return a list of templates that could have one or more elements.
        Args:
            name: name of the template to find.
            hv: the ID of the hypervisor to search the template in
        Returns:
            A list of templates object. If hv is None will return all the
            templates matching the name if every hypervisor type. Otherwise
            if name is None will return all templates of an hypervisor.
        Raises:
            ValidationError: if name and hv are None
        """
        if len(self.templates) <= 0:
            self.get_hypervisors()
        if name is not None and hv is not None:
            template_list = filter(
                lambda x: name in x.descr and x.hypervisor == self.hypervisors[hv], self.templates
            )
        elif name is not None and hv is None:
            template_list = filter(
                lambda x: name in x.descr, self.templates
            )
        elif name is None and hv is not None:
            template_list = filter(
                lambda x: x.hypervisor == self.hypervisors[hv], self.templates
            )
        else:
            raise Exception('Error, no pattern defined')
        if  sys.version_info.major < (3):
            return template_list
        else:
            return(list(template_list))

    def get_vm(self, pattern=None):
        if len(self.vmlist) <= 0:
            self.get_servers()
        if pattern is None:
            return self.vmlist
        else:
            return self.vmlist.find(pattern)

    def get_ip_by_vm(self, vm):
        self.get_ip()  # call get ip list to create the internal list of IPs.
        vm_id = self.get_vm(vm)[0].sid
        for ip in self.iplist:
            if ip.serverid == vm_id:
                return ip
        return 'IPNOTFOUND'

    def purchase_ip(self, debug=False):
        """
        Return an ip object representing a new bought IP
        @param debug [Boolean] if true, request and response will be printed
        @return (Ip): Ip object
        """
        json_scheme = self.gen_def_json_scheme('SetPurchaseIpAddress')
        json_obj = self.call_method_post(method='SetPurchaseIpAddress', json_scheme=json_scheme, debug=debug)
        try:
            ip = Ip()
            ip.ip_addr = json_obj['Value']['Value']
            ip.resid = json_obj['Value']['ResourceId']
            return ip
        except:
            raise Exception('Unknown error retrieving IP.')

    def purchase_vlan(self, vlan_name, debug=False):
        """
        Purchase a new VLAN.
        :param debug: Log the json response if True
        :param vlan_name: String representing the name of the vlan (virtual switch)
        :return: a Vlan Object representing the vlan created
        """
        vlan_name = {'VLanName': vlan_name}
        json_scheme = self.gen_def_json_scheme('SetPurchaseVLan', vlan_name)
        json_obj = self.call_method_post(method="SetPurchaseVLan", json_scheme=json_scheme)
        if debug is True:
            self.logger.debug(json_obj)
        if json_obj['Success'] is False:
            raise Exception("Cannot purchase new vlan.")
        vlan = Vlan()
        vlan.name = json_obj['Value']['Name']
        vlan.resource_id = json_obj['Value']['ResourceId']
        vlan.vlan_code = json_obj['Value']['VlanCode']
        return vlan

    def remove_vlan(self, vlan_resource_id):
        """
        Remove a VLAN
        :param vlan_resource_id:
        :return:
        """
        vlan_id = {'VLanResourceId': vlan_resource_id}
        json_scheme = self.gen_def_json_scheme('SetRemoveVLan', vlan_id)
        json_obj = self.call_method_post(method='SetRemoveVLan', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False

    def get_vlan(self, vlan_name=None):
        json_scheme = self.gen_def_json_scheme('GetPurchasedVLans')
        json_obj = self.call_method_post(method='GetPurchasedVLans', json_scheme=json_scheme)
        if vlan_name is not None:
            raw_vlans = filter(lambda x: vlan_name in x['Name'], json_obj['Value'])
        else:
            raw_vlans = json_obj['Value']
        vlans = []
        for raw_vlan in raw_vlans:
            v = Vlan()
            v.name = raw_vlan['Name']
            v.vlan_code = raw_vlan['VlanCode']
            v.resource_id = raw_vlan['ResourceId']
            vlans.append(v)
        return vlans

    def remove_ip(self, ip_id):
        """
        Delete an Ip from the boughs ip list
        @param (str) ip_id: a string representing the resource id of the IP
        @return: True if json method had success else False
        """
        ip_id = '    "IpAddressResourceId": %s' % ip_id
        json_scheme = self.gen_def_json_scheme('SetRemoveIpAddress', ip_id)
        json_obj = self.call_method_post(method='SetRemoveIpAddress', json_scheme=json_scheme)
        pprint(json_obj)
        return True if json_obj['Success'] is True else False

    def get_ip(self):
        """
        Retrieve a complete list of bought ip address related only to PRO Servers.
        It create an internal object (Iplist) representing all of the ips object
        iterated form the WS.
        @param: None
        @return: None
        """
        json_scheme = self.gen_def_json_scheme('GetPurchasedIpAddresses')
        json_obj = self.call_method_post(method='GetPurchasedIpAddresses ', json_scheme=json_scheme)
        self.iplist = IpList()
        for ip in json_obj['Value']:
            r = Ip()
            r.ip_addr = ip['Value']
            r.resid = ip['ResourceId']
            r.serverid = ip['ServerId'] if 'None' not in str(ip['ServerId']) else None
            self.iplist.append(r)

    def delete_vm(self, server=None, server_id=None):
        self.logger.debug('%s: Deleting: %s' % (self.__class__.__name__, server))
        sid = server_id if server_id is not None else server.sid
        self.logger.debug('%s: Deleting SID: %s' % (self.__class__.__name__, sid))
        if sid is None:
            raise Exception('NoServerSpecified')
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerDeletion', dict(ServerId=sid))
        json_obj = self.call_method_post(method='SetEnqueueServerDeletion', json_scheme=json_scheme)
        print('Deletion enqueued successfully for server_id: %s' % sid)
        return True if json_obj['Success'] is 'True' else False

    def get_jobs(self):
        json_scheme = self.gen_def_json_scheme('GetJobs')
        return self.call_method_post(method='GetJobs', json_scheme=json_scheme)

    def find_job(self, vm_name):
        jobs_list = self.get_jobs()
        if jobs_list['Value'] is None:
            _i = 0
            while jobs_list['Value'] is not None:
                _i += 1
                jobs_list = self.get_jobs()
                if _i > 10:
                    return 'JOBNOTFOUND'
        if len(jobs_list['Value']) <= 0:
            return 'JOBNOTFOUND'
        for job in jobs_list['Value']:
            if vm_name in job['ServerName']:
                return job
        return 'JOBNOTFOUND'

    def get_virtual_datacenter(self):
        json_scheme = self.gen_def_json_scheme('GetVirtualDatacenter')
        json_obj = self.call_method_post(method='GetVirtualDatacenter', json_scheme=json_scheme)
        return json_obj

    def get_server_detail(self, server_id):
        json_scheme = self.gen_def_json_scheme('GetServerDetails', dict(ServerId=server_id))
        json_obj = self.call_method_post(method='GetServerDetails', json_scheme=json_scheme)
        return json_obj['Value']

    def attach_vlan(self, network_adapter_id, vlan_resource_id, ip=None, subnet_mask=None, gateway=None):
        if gateway is not None:
            additional_fields = {
                "VLanRequest": {
                    "NetworkAdapterId": network_adapter_id,
                    "SetOnVirtualMachine": "true",
                    "VLanResourceId": vlan_resource_id,
                    "PrivateIps": [{
                        "GateWay": gateway,
                        "IP": ip,
                        "SubNetMask": subnet_mask
                    }]
                }
            }
        else:
            additional_fields = {
                "VLanRequest": {
                    "NetworkAdapterId": network_adapter_id,
                    "SetOnVirtualMachine": "false",
                    "VLanResourceId": vlan_resource_id,
                    "PrivateIps": [{
                        "GateWay": None,
                        "IP": None,
                        "SubNetMask": None
                    }]
                }
            }
        json_scheme = self.gen_def_json_scheme('SetEnqueueAssociateVLan', method_fields=additional_fields)
        json_obj = self.call_method_post(method='SetEnqueueAssociateVLan', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False

    def detach_vlan(self, network_adapter_id, vlan_resource_id):
        vlan_request = {
            "VLanRequest": {
                "NetworkAdapterId": network_adapter_id,
                "SetOnVirtualMachine": "false",
                "VLanResourceId": vlan_resource_id
            }
        }
        json_scheme = self.gen_def_json_scheme('SetEnqueueDeassociateVLan', method_fields=vlan_request)
        json_obj = self.call_method_post(method='SetEnqueueDeassociateVLan', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False

    def create_snapshot(self, dc, server_id=None):
        sid = CloudInterface(dc).get_server_detail(server_id)
        if sid['HypervisorType'] is not 4:
            snapshot_request = {
                "Snapshot": {
                    "ServerId": server_id,
                    "SnapshotOperationTypes": "Create"
                }
            }
            json_scheme = self.gen_def_json_scheme('SetEnqueueServerSnapshot', method_fields=snapshot_request)
            json_obj = self.call_method_post(method='SetEnqueueServerSnapshot', json_scheme=json_scheme)
            return True if json_obj['Success'] is True else False

    def restore_snapshot(self, server_id=None):
        snapshot_request = {
            "Snapshot": {
                "ServerId": server_id,
                "SnapshotOperationTypes": "Restore"
            }
        }
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerSnapshot', method_fields=snapshot_request)
        json_obj = self.call_method_post(method='SetEnqueueServerSnapshot', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False

    def delete_snapshot(self, server_id=None):
        snapshot_request = {
            "Snapshot": {
                "ServerId": server_id,
                "SnapshotOperationTypes": "Delete"
            }
        }
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerSnapshot', method_fields=snapshot_request)
        json_obj = self.call_method_post(method='SetEnqueueServerSnapshot', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False

    def archive_vm(self, dc, server_id=None):
        sid = CloudInterface(dc).get_server_detail(server_id)
        if sid['HypervisorType'] is not 4:
            archive_request = {
                "ArchiveVirtualServer": {
                    "ServerId": server_id
                }
            }
            json_scheme = self.gen_def_json_scheme('ArchiveVirtualServer', method_fields=archive_request)
            json_obj = self.call_method_post(method='ArchiveVirtualServer', json_scheme=json_scheme)
            return True if json_obj['Success'] is True else False

    def restore_vm(self, server_id=None, cpu_qty=None, ram_qty=None):
        restore_request = {
            "SetEnqueueServerRestore": {
                "ServerId": server_id,
                "CPUQuantity ": cpu_qty,
                "RAMQuantity ": ram_qty
            }
        }
        json_scheme = self.gen_def_json_scheme('SetEnqueueServerRestore', method_fields=restore_request)
        json_obj = self.call_method_post(method='SetEnqueueServerRestore', json_scheme=json_scheme)
        return True if json_obj['Success'] is True else False



from ArubaCloud.base import Request


class BaseReverseDns(Request):
    def __init__(self, *args, **kwargs):
        super(BaseReverseDns, self).__init__(*args, **kwargs)
        # noinspection PyPep8Naming

    def commit(self):
        pass



from ArubaCloud.ReverseDns.Requests import *
from ArubaCloud.base import ArubaCloudService


class ReverseDns(ArubaCloudService):
    def __init__(self, ws_uri, username, password):
        super(ReverseDns, self).__init__(ws_uri, username, password)

    def _call(self, method, *args, **kwargs):
        return method(Username=self.username, Password=self.password, uri=self.ws_uri, *args, **kwargs)

    def get(self, addresses):
        """
        :type addresses: list[str]
        :param addresses: (list[str]) List of addresses to retrieve their reverse dns
        Retrieve the current configured ReverseDns entries
        :return: (list) List containing the current ReverseDns Addresses
        """
        request = self._call(GetReverseDns, IPs=addresses)
        response = request.commit()
        return response['Value']

    def set(self, address, host_name):
        """
        Assign one or more PTR record to a single IP Address
        :type address: str
        :type host_name: list[str]
        :param address: (str) The IP address to configure
        :param host_name: (list[str]) The list of strings representing PTR records
        :return: (bool) True in case of success, False in case of failure
        """
        request = self._call(SetEnqueueSetReverseDns, IP=address, Hosts=host_name)
        response = request.commit()
        return response['Success']

    def reset(self, address=None):
        """
        Remove all PTR records from the given address
        :type address: List[str]
        :param address: (List[str]) The IP Address to reset
        :return: (bool) True in case of success, False in case of failure
        """
        request = self._call(SetEnqueueResetReverseDns, IPs=address)
        response = request.commit()
        return response['Success']



class VMList(list):
    def __init__(self, *args, **kwargs):
        super(VMList, self).__init__(*args)
        self.last_search_result = []

    def find(self, name):
        """
        Return a list of subset of VM that match the pattern name
        @param name (str): the vm name of the virtual machine
        @param name (Obj): the vm object that represent the virtual
                           machine (can be Pro or Smart)
        @return (list): the subset containing the serach result.
        """
        if name.__class__ is 'base.Server.Pro' or name.__class__ is 'base.Server.Smart':
            # print('DEBUG: matched VM object %s' % name.__class__)
            pattern = name.vm_name
        else:
            # print('DEBUG: matched Str Object %s' % name.__class__)
            pattern = name
        # 14/06/2013: since this method is called within a thread and I wont to pass the return objects with queue or
        # call back, I will allocate a list inside the Interface class object itself, which contain all of the vm found
        # 02/11/2015: this must be changed ASAP! it's a mess this way... what was I thinking??
        self.last_search_result = [vm for vm in self if pattern in vm.vm_name]
        return self.last_search_result

    def show(self):
        for vm in self:
            print(vm)

    def find_ip(self, ip):
        f = None
        if ip.__class__ is 'base.Ip.Ip':
            # logger.debug('DEBUG: matched IP Object: %s' % ip.__class__)
            pattern = ip.ip_addr
        else:
            # logger.debug('DEBUG: matched Str Object: %s' % ip.__class__)
            pattern = ip
        for vm in self:
            if vm.__class__.__name__ is 'Smart':
                if pattern == vm.ip_addr:
                    f = vm
            else:
                if pattern == vm.ip_addr.ip_addr:
                    f = vm
        return f


class VM(object):
    vm_name = None
    cpu_qty = None
    ram_qty = None
    status = None
    sid = None
    datacenter_id = None
    auth = None
    admin_password = None
    wcf_baseurl = None
    template_id = None
    hd_total_size = None
    hd_qty = None

    def __init__(self, interface):
        super(VM, self).__init__()
        self.interface = interface

    def poweroff(self, debug=False):
        data = dict(
            ServerId=self.sid
        )
        json_scheme = self.interface.gen_def_json_scheme('SetEnqueueServerPowerOff', data)
        json_obj = self.interface.call_method_post('SetEnqueueServerPowerOff', json_scheme=json_scheme, debug=debug)
        return True if json_obj['Success'] is True else False

    def poweron(self, debug=False):
        data = dict(
            ServerId=self.sid
        )
        json_scheme = self.interface.gen_def_json_scheme('SetEnqueueServerStart', data)
        json_obj = self.interface.call_method_post('SetEnqueueServerStart', json_scheme=json_scheme, debug=debug)
        return True if json_obj['Success'] is 'True' else False

    def reinitialize(self, admin_password=None, debug=False, ConfigureIPv6=False, OSTemplateID=None):
        """
        Reinitialize a VM.
        :param admin_password: Administrator password.
        :param debug: Flag to enable debug output.
        :param ConfigureIPv6: Flag to enable IPv6 on the VM.
        :param OSTemplateID: TemplateID to reinitialize the VM with.
        :return: True in case of success, otherwise False
        :type admin_password: str
        :type debug: bool
        :type ConfigureIPv6: bool
        :type OSTemplateID: int
        """
        data = dict(
            AdministratorPassword=admin_password,
            ServerId=self.sid,
            ConfigureIPv6=ConfigureIPv6
        )
        if OSTemplateID is not None:
            data.update(OSTemplateID=OSTemplateID)
        assert data['AdministratorPassword'] is not None, 'Error reinitializing VM: no admin password specified.'
        assert data['ServerId'] is not None, 'Error reinitializing VM: no Server Id specified.'
        json_scheme = self.interface.gen_def_json_scheme('SetEnqueueReinitializeServer', method_fields=data)
        json_obj = self.interface.call_method_post('SetEnqueueReinitializeServer', json_scheme=json_scheme, debug=debug)
        return True if json_obj['Success'] is 'True' else False

    def edit_cpu(self, cpu_qty, debug=False):
        raise NotImplemented()

    def edit_ram(self, ram_qty, debug=False):
        raise NotImplemented()

    def add_virtual_disk(self, *args, **kwargs):
        raise NotImplemented()

    def remove_virtual_disk(self, *args, **kwargs):
        raise NotImplemented()

    def edit_virtual_disk_size(self, *args, **kwargs):
        raise NotImplemented()



from ._BaseReverseDns import BaseReverseDns


# noinspection PyPep8Naming
class GetReverseDns(BaseReverseDns):
    def __init__(self, *args, **kwargs):
        try:
            self.IPs = kwargs.pop('IPs')
        except KeyError:
            # IPs parameter is not filled
            self.IPs = []
        super(GetReverseDns, self).__init__(*args, **kwargs)

    def commit(self):
        return self._commit()



class Template(object):
    descr = None
    template_id = ''
    id_code = None
    hypervisor = None
    name = None
    enabled = None

    class ResourceBounds(object):
        def __init__(self):
            self.max_cpu = None
            self.max_memory = None
            self.hdd0 = None
            self.hdd1 = None
            self.hdd2 = None
            self.hdd3 = None

    def __init__(self, hypervisor):
        self.hypervisor = hypervisor
        self.resource_bounds = self.ResourceBounds()

    def __str__(self):
        msg = 'Template Name: %s\n' % self.descr
        msg += ' -> Hypervisor: %s\n' % self.hypervisor
        msg += ' -> IdCode: %s\n' % self.id_code
        msg += ' -> Id: %s\n' % self.template_id
        msg += ' -> Enabled: %s\n' % self.enabled
        return msg

    def __repr__(self):
        return 'Template Name: %s, Hypervisor: %s, Id: %s, Enabled: %s' % (
            self.descr, self.hypervisor, self.template_id, self.enabled
        )



from ArubaCloud.base.Errors import OperationNotPermitted, MalformedJsonRequest
from ArubaCloud.base.vm import VM
from ArubaCloud.objects import Ip
from ArubaCloud.objects import VirtualDiskOperation


class Smart(VM):
    def __init__(self, interface, sid):
        super(Smart, self).__init__(interface)
        self.cltype = 'smart'
        self.sid = sid
        self.ip_addr = Ip()
        self.package = None

    def __str__(self):
        msg = super(Smart, self).__str__()
        msg += ' -> IPAddr: %s\n' % self.ip_addr
        return msg

    def upgrade_vm(self, package_id, debug=False):
        if not self.status == 2:
            raise OperationNotPermitted("Cannot edit resources in the current state of the VM. Turn off the VM first.")
        if package_id > 4 or package_id <= 0:
            return MalformedJsonRequest("Packages available are: 1, 2, 3, 4.")
        method_json = {
            "Server": {
                "ServerId": self.sid,
                "SmartVMWarePackageID": package_id
            }
        }
        json_obj = self.interface.call_method_post(method='SetEnqueueServerUpdate',
                                                   json_scheme=self.interface.gen_def_json_scheme(
                                                       req='SetEnqueueServerUpdate',
                                                       method_fields=method_json
                                                   ),
                                                   debug=debug)
        if debug is True:
            print(json_obj)
        return True if json_obj['Success'] is True else False


class Pro(VM):
    def __init__(self, interface, sid):
        super(Pro, self).__init__(interface)
        self.cltype = 'pro'
        self.sid = sid
        self.ip_addr = Ip()
        # Load HD information
        self.hds = []
        _hds = self.interface.get_server_detail(self.sid)['VirtualDisks']
        for hd in _hds:
            self.hds.append(hd)

    """
    Methods specific for Pro VM
    """

    def _commit_compute_resources(self):
        method_json = {
            "ServerId": self.sid,
            "CpuQuantity": self.cpu_qty,
            "RamQuantity": self.ram_qty,
            "RestartAfterExecuted": 'true'
        }
        json_obj = self.interface.call_method_post(method='SetEnqueueHardwareUpdate',
                                                   json_scheme=self.interface.gen_def_json_scheme(
                                                       req='SetEnqueueHardwareUpdate',
                                                       method_fields=method_json)
                                                   )
        return json_obj

    def edit_cpu(self, cpu_qty, debug=False):
        if not self.status == 2:
            raise OperationNotPermitted("Cannot edit resources in the current state of the VM.")
        self.cpu_qty = cpu_qty
        json_obj = self._commit_compute_resources()
        if debug is True:
            print(json_obj)
        return True if json_obj['Success'] is True else False

    def edit_ram(self, ram_qty, debug=False):
        if not self.status == 2:
            raise OperationNotPermitted("Cannot edit resources in the current state of the VM.")
        self.ram_qty = ram_qty
        json_obj = self._commit_compute_resources()
        if debug is True:
            print(json_obj)
        return True if json_obj['Success'] is True else False

    def add_virtual_disk(self, size, debug=False):
        if not self.status == 2:
            raise OperationNotPermitted("Cannot edit resources in the current state of the VM.")
        if len(self.hds) > 3:
            raise ValueError("Cannot create more than 3 disks per VM.")
        virtual_disk_operation = VirtualDiskOperation

        method_json = {
            'ServerId': self.sid,
            'Disk': {
                'CustomVirtualDiskPath': None,
                'Size': size,
                'VirtualDiskType': len(self.hds),  # increment the counter of current present disks
                'VirtualDiskUpdateType': virtual_disk_operation.create
            }
        }
        json_obj = self.interface.call_method_post(method='SetEnqueueVirtualDiskManage',
                                                   json_scheme=self.interface.gen_def_json_scheme(
                                                       req='SetEnqueueVirtualDiskManage',
                                                       method_fields=method_json)
                                                   )
        if debug is True:
            print(json_obj)
        return True if json_obj['Success'] is True else False

    def resize_virtual_disk(self, size, debug=False):
            if not self.status == 2:
                raise OperationNotPermitted("Cannot edit resources in the current state of the VM.")
            if self.hd_total_size > size:
                raise OperationNotPermitted("Cannot decrease size of the disk")
            virtual_disk_operation = VirtualDiskOperation

            method_json = {
                'ServerId': self.sid,
                'Disk': {
                    'CustomVirtualDiskPath': None,
                    'Size': size,
                    'VirtualDiskType': self.hds,
                    'VirtualDiskUpdateType': virtual_disk_operation.resize
                }
            }
            json_obj = self.interface.call_method_post(method='SetEnqueueVirtualDiskManage',
                                                       json_scheme=self.interface.gen_def_json_scheme(
                                                           req='SetEnqueueVirtualDiskManage',
                                                           method_fields=method_json)
                                                       )
            if debug is True:
                print(json_obj)
            return True if json_obj['Success'] is True else False

    def remove_virtual_disk(self, virtual_disk_id, debug=False):
        if not self.status == 2:
            raise OperationNotPermitted("Cannot edit resources in the current state of the VM.")
        virtual_disk_operation = VirtualDiskOperation

        method_json = {
            'ServerId': self.sid,
            'Disk': {
                'CustomVirtualDiskPath': None,
                'Size': 0,
                'VirtualDiskType': virtual_disk_id,
                'VirtualDiskUpdateType': virtual_disk_operation.delete
            }
        }
        json_obj = self.interface.call_method_post(method='SetEnqueueVirtualDiskManage',
                                                   json_scheme=self.interface.gen_def_json_scheme(
                                                       req='SetEnqueueVirtualDiskManage',
                                                       method_fields=method_json)
                                                   )
        if debug is True:
            print(json_obj)
        return True if json_obj['Success'] is True else False

    def __str__(self):
        msg = super(Pro, self).__str__()
        msg += ' -> IPAddr: %s\n' % self.ip_addr
        return msg



import json
from abc import ABCMeta
from collections import OrderedDict

from ArubaCloud.base.Errors import ValidationError


class Creator(object):
    __metaclass__ = ABCMeta
    json_msg = OrderedDict()

    def get_raw(self):
        return self.json_msg

    def get_json(self):
        return json.dumps(self.json_msg)

    def commit(self, url, debug=False):
        from ArubaCloud.helper import Http
        url = '{}/{}'.format(url, 'SetEnqueueServerCreation')
        headers = {'Content-Type': 'application/json', 'Content-Length': str(len(self.get_json()))}
        response = Http.post(url=url, data=self.get_json(), headers=headers)
        if response.status_code != 200:
            print(response.content)
            return False
        parsed_response = json.loads(response.content)
        if debug is True:
            print(parsed_response)
        if parsed_response["Success"]:
            return True
        return False


class ProVmCreator(Creator):
    def __init__(self, name, admin_password, template_id, auth_obj, note=None):
        self.name = name
        self.admin_password = admin_password
        self.template_id = template_id
        self.auth = auth_obj
        self.note = 'Create by pyArubaCloud' if note is None else note
        self.ethernet_counter = 0
        self.virtual_disk_counter = 0
        self.json_msg = {
            'ApplicationId': 'SetEnqueueServerCreation',
            'RequestId': 'SetEnqueueServerCreation',
            'SessionId': '',
            'Password': auth_obj.password,
            'Username': self.auth.username,
            'Server': {
                'AdministratorPassword': self.admin_password,
                'CPUQuantity': 0,
                'Name': self.name,
                'NetworkAdaptersConfiguration': [],
                'Note': self.note,
                'OSTemplateId': self.template_id,
                'RAMQuantity': 0,
                'VirtualDisks': []
            }
        }

    def add_public_ip(self, public_ip_address_resource_id, primary_ip_address='true'):
        if self.ethernet_counter > 0:
            raise ValueError("Public IP Address must be bind to first Ethernet Interface.")
        network_adapter = {
            'NetworkAdapterType': self.ethernet_counter,
            'PublicIpAddresses': [{
                'PrimaryIPAddress': primary_ip_address,
                'PublicIpAddressResourceId': public_ip_address_resource_id
            }]
        }
        try:
            self.json_msg['Server']['NetworkAdaptersConfiguration'].append(network_adapter)
            self.ethernet_counter += 1
            return True
        except KeyError:
            return False

    def add_private_vlan(self, gateway, ip_address, private_vlan_resource_id, subnet_mask):
        if self.ethernet_counter > 2:
            raise ValueError("Cannot create more than 3 ethernet interface per VM.")
        network_adapter = {
            'NetworkAdapterType': self.ethernet_counter,
            'PrivateVLan': {
                'Gateway': gateway,
                'IPAddress': ip_address,
                'PrivateVLanResourceId': private_vlan_resource_id,
                'SubNetMask': subnet_mask
            }
        }
        try:
            self.json_msg['Server']['NetworkAdaptersConfiguration'].append(network_adapter)
            self.ethernet_counter += 1
            return True
        except KeyError:
            return False

    def add_virtual_disk(self, size):
        if self.virtual_disk_counter > 3:
            raise ValueError("Cannot create more than 4 disk.")
        if size > 500:
            raise ValueError("MaxSize per Disk: 500 GB.")
        virtual_disk = {
            "Size": size,
            "VirtualDiskType": self.virtual_disk_counter
        }
        try:
            self.json_msg['Server']['VirtualDisks'].append(virtual_disk)
            self.virtual_disk_counter += 1
            return True
        except KeyError:
            return False

    def set_cpu_qty(self, cpu_qty):
        self.json_msg['Server']['CPUQuantity'] = cpu_qty

    def set_ram_qty(self, ram_qty):
        self.json_msg['Server']['RAMQuantity'] = ram_qty
        
    def set_ssh_key(self, public_key_path):
        with open(public_key_path, 'r') as content_file:
            content = content_file.read()
        self.json_msg['Server']['SshKey'] = content
        self.json_msg['Server']['SshPasswordAuthAllowed'] = True


class SmartVmCreator(Creator):
    def __init__(self, name, admin_password, template_id, auth_obj, note=None):
        self.name = name
        self.admin_password = admin_password
        self.template_id = template_id
        self.auth_obj = auth_obj
        self.note = 'Create by pyArubaCloud' if note is None else note
        self.json_msg = {
            'ApplicationId': 'SetEnqueueServerCreation',
            'RequestId': 'SetEnqueueServerCreation',
            'Username': self.auth_obj.username,
            'Password': self.auth_obj.password,
            'Server': {
                'AdministratorPassword': self.admin_password,
                'Name': self.name,
                'SmartVMWarePackageID': 1,
                'Note': self.note,
                'OSTemplateId': self.template_id
            }
        }

    def set_type(self, size='small'):
        """
        Define the size of the VM. Size available:
         - small
         - medium
         - large
         - extralarge
        :param size: str()
        :return:
        """
        sizes = {'small': 1,
                 'medium': 2,
                 'large': 3,
                 'extralarge': 4}
        self.json_msg['Server']['SmartVMWarePackageID'] = sizes[size]
        return True
    
    def set_ssh_key(self, public_key_path):
        with open(public_key_path, 'r') as content_file:
            content = content_file.read()
        self.json_msg['Server']['SshKey'] = content
        self.json_msg['Server']['SshPasswordAuthAllowed'] = True


class Vlan(object):
    name = None
    resource_id = None
    vlan_code = None

    def __repr__(self):
        return 'Vlan(name={}, resource_id={}, vlan_code={})'.format(self.name, self.resource_id, self.vlan_code)

    def __str__(self):
        return 'Vlan(name={}, resource_id={}, vlan_code={})'.format(self.name, self.resource_id, self.vlan_code)


class NetworkAdapter(object):
    ip_addresses = []
    id = None
    mac_address = None
    network_adapter_type = None
    server_id = None
    vlan_id = None


class IpList(list):
    def __init__(self, *args):
        super(IpList, self).__init__(*args)

    def show(self):
        for elem in self:
            print(elem)

    def find(self, vm_name=None, ip_addr=None, resid=None):
        # more defensive checks, just to have fun...
        params = locals()
        pattern = {}
        ip = None
        for _item in params:
            if isinstance(params[_item], str) or isinstance(params[_item], int):
                pattern['criteria'] = _item
                pattern['value'] = params[_item]
        for elem in self:
            if not hasattr(elem, pattern['criteria']):
                raise ValidationError('The criteria specified does not exists: %s' % (pattern['criteria']))
            mtc = getattr(elem, pattern['criteria'])
            if mtc == pattern['value']:
                # we will return this object
                ip = elem
        # return the ip object which match the criteria
        assert (ip.__class__.__name__ is 'Ip' or ip is None), 'the returning object is not as expected.'
        return ip


class Ip(object):
    ip_addr = None
    resid = None
    serverid = None

    def __init__(self):
        pass

    def is_mapped(self):
        return True if self.serverid is not None else False


class VirtualDiskOperation(object):
    resize = 1
    create = 2
    delete = 3


class VirtualDisk(object):
    primary_disk_type = 3
    additional_disk1_type = 7
    additional_disk2_type = 8
    additional_disk3_type = 9
    primary_disk_id = 0
    additional_disk1_id = 1
    additional_disk2_id = 2
    additional_disk3_id = 3



from ._BaseReverseDns import BaseReverseDns


class SetEnqueueSetReverseDns(BaseReverseDns):
    def __init__(self, *args, **kwargs):
        """
        Assign one or more PTR record to a single IP Address
        :type IP: str
        :type Host: list[str]
        :param IP: (str) The IP address to configure
        :param Host: (list[str]) The list of strings representing PTR records
        """
        try:
            self.IP = kwargs.pop('IP')
        except KeyError:
            raise Exception('IP parameter cannot be null.')
        try:
            self.Hosts = kwargs.pop('Hosts')
        except KeyError:
            raise Exception('Hosts parameter cannot be null.')
        super(SetEnqueueSetReverseDns, self).__init__(*args, **kwargs)

    def commit(self):
        return self._commit()



import logging


class ArubaLog(object):
    def __init__(self, level=logging.DEBUG, dst_file='debug.log', name=__name__, log_to_file=False,
                 log_to_stdout=True):
        """
        Initialize Logging System

        :param level: Debugging level.
        :param dst_file: Log destination file.
        :param name: the name of the debugger.
        :param log_to_file: boolean, True in case logs must be placed on file too.
        """
        self.level = level
        self.dst_file = dst_file
        self.log = logging.getLogger(name)
        self.log.setLevel(level)
        self.name = name

        formatter = logging.Formatter("%(asctime)s %(threadName)-11s %(levelname)-10s %(message)s")

        if log_to_file is True:
            file_handler = logging.FileHandler(dst_file, "a")
            file_handler.setLevel(level)
            file_handler.setFormatter(formatter)
            self.log.addHandler(file_handler)

        elif log_to_stdout is True:
            stream_handler = logging.StreamHandler()
            stream_handler.setLevel(level)
            stream_handler.setFormatter(formatter)
            self.log.addHandler(stream_handler)
        else:
            raise ValueError("No Stream Handler Specified.")

    def debug(self, msg):
        self.log.debug(msg)

    def critical(self, msg):
        self.log.critical(msg)

    def warning(self, msg):
        self.log.warning(msg)

    def info(self, msg):
        self.log.info(msg)

    def error(self, error):
        self.log.error(error)



from GetReverseDns import GetReverseDns
from SetEnqueueSetReverseDns import SetEnqueueSetReverseDns
from SetEnqueueResetReverseDns import SetEnqueueResetReverseDns

__all__ = ['GetReverseDns', 'SetEnqueueResetReverseDns', 'SetEnqueueSetReverseDns']




import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="pyArubaCloud",
    version="0.7.4",
    author="Aruba S.p.A.",
    author_email="luca.lasagni@staff.aruba.it",
    description="Python Interface to interact with ArubaCloud IaaS Service.",
    license="MIT",
    keywords="arubacloud",
    url="https://github.com/Arubacloud/pyArubaCloud",
    packages=['ArubaCloud', 'ArubaCloud.base', 'ArubaCloud.helper', 'ArubaCloud.objects.VmTypes', 'ArubaCloud.objects'],
    long_description="Python Interface to interact with ArubaCLoud IaaS Service.",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=['requests>=2.1.9', 'jsonpickle', 'codeclimate-test-reporter']
)



import json
from abc import ABCMeta, abstractmethod

from ArubaCloud.helper import Http
import jsonpickle


class JsonInterfaceBase(object):
    __metaclass__ = ABCMeta

    def __init__(self):
        pass

    def gen_def_json_scheme(self, req, method_fields=None):
        """
        Generate the scheme for the json request.
        :param req: String representing the name of the method to call
        :param method_fields: A dictionary containing the method-specified fields
        :rtype : json object representing the method call
        """
        json_dict = dict(
            ApplicationId=req,
            RequestId=req,
            SessionId=req,
            Password=self.auth.password,
            Username=self.auth.username
        )
        if method_fields is not None:
            json_dict.update(method_fields)
        self.logger.debug(json.dumps(json_dict))
        return json.dumps(json_dict)

    def call_method_post(self, method, json_scheme, debug=False):
        url = '{}/{}'.format(self.wcf_baseurl, method)
        headers = {'Content-Type': 'application/json', 'Content-Length': str(len(json_scheme))}
        response = Http.post(url=url, data=json_scheme, headers=headers)
        parsed_response = json.loads(response.content.decode('utf-8'))
        if response.status_code != 200:
            from ArubaCloud.base.Errors import MalformedJsonRequest
            raise MalformedJsonRequest("Request: {}, Status Code: {}".format(json_scheme, response.status_code))
        if parsed_response['Success'] is False:
            from ArubaCloud.base.Errors import RequestFailed
            raise RequestFailed("Request: {}, Response: {}".format(json_scheme, parsed_response))
        if debug is True:
            msg = "Response Message: {}\nHTTP Status Code: {}".format(parsed_response, response.status_code)
            self.logger.debug(msg)
            print(msg)
        return parsed_response


class IRequest(object):
    __metaclass__ = ABCMeta

    def __init__(self):
        pass

    @abstractmethod
    def _commit(self):
        raise NotImplementedError

    @abstractmethod
    def commit(self):
        raise NotImplementedError


# noinspection PyPep8Naming
class Request(IRequest):
    def __init__(self, logger=None, Username=str(), Password=str(), SessionId=None, ApplicationId=None, RequestId=None,
                 uri=None):
        """
        :type logger: ArubaLog
        :type Username: str
        :type Password: str
        :type SessionId: str
        :type ApplicationId: str
        :type RequestId: str
        :type uri: str
        :param logger: Logger object
        :param Username: ArubaCloud Service Login Username
        :param Password: ArubaCloud Service Login Password
        :param SessionId: Can be Null, otherwise the current SessionId
        :param ApplicationId: Same as RequestId
        :param RequestId: The name of the Request
        :param uri: WCF base URI
        """
        super(Request, self).__init__()
        self.logger = logger
        self.Username = Username
        self.Password = Password
        self.SessionId = SessionId if SessionId is not None else self.__class__.__name__
        self.ApplicationId = ApplicationId if ApplicationId is not None else self.__class__.__name__
        self.RequestId = RequestId if RequestId is not None else self.__class__.__name__
        self.uri = uri

    def _commit(self):
        """
        :return: (dict) Response object content
        """
        assert self.uri is not None, Exception("BadArgument: uri property cannot be None")
        url = '{}/{}'.format(self.uri, self.__class__.__name__)
        serialized_json = jsonpickle.encode(self, unpicklable=False, )
        headers = {'Content-Type': 'application/json', 'Content-Length': str(len(serialized_json))}
        response = Http.post(url=url, data=serialized_json, headers=headers)
        if response.status_code != 200:
            from ArubaCloud.base.Errors import MalformedJsonRequest
            raise MalformedJsonRequest("Request: {}, Status Code: {}".format(serialized_json, response.status_code))
        content = jsonpickle.decode(response.content.decode("utf-8"))
        if content['ResultCode'] == 17:
            from ArubaCloud.base.Errors import OperationAlreadyEnqueued
            raise OperationAlreadyEnqueued("{} already enqueued".format(self.__class__.__name__))
        if content['Success'] is False:
            from ArubaCloud.base.Errors import RequestFailed
            raise RequestFailed("Request: {}, Response: {}".format(serialized_json, response.content))
        return content

    @abstractmethod
    def commit(self):
        raise NotImplementedError("commit method must be implemented in the real request implementation class")

    def __getstate__(self):
        """
        Internal method to remove non serializable object before the object serialization
        :return: (Request) A copy of the state of the object after removing unwanted fields
        """
        state = self.__dict__.copy()
        del state['logger']
        del state['uri']
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)


class ArubaCloudService(object):
    def __init__(self, ws_uri, username, password, token=None):
        """
        :type ws_uri: str
        :type username: str
        :type password: str
        :type token: str
        :param ws_uri: WsEndUser URI
        :param username: ArubaCloud Service Username
        :param password: ArubaCloud Service Password
        :param token: Nullable, Token to be used instead of password
        """
        self.ws_uri = ws_uri
        self.username = username
        self.password = password
        self.token = token

    def login(self):
        raise NotImplemented('Not implemented yet...')

    def _call(self, method, *args, **kwargs):
        raise NotImplementedError


class Auth(object):
    username = None
    password = None
    token = None
    enc_pwd = None

    def __init__(self, username=None, password=None):
        self.username = username
        self.password = password



import requests


class Http(object):
    @staticmethod
    def _log_request(logger, data=None, headers=None):
        if logger is not None:
            if headers is not None:
                for header in headers:
                    logger.debug('request header: %s: %s', header, headers[header])
            if data is not None:
                logger.debug('request data:\n %s', data)

    @staticmethod
    def _log_response(logger, response):
        if logger is not None:
            logger.debug('[%d] %s', response.status_code, response.text)

    @staticmethod
    def get(url, data=None, logger=None, **kwargs):
        if logger is not None:
            Http._log_request(logger, data=data, headers=kwargs.get('headers', None))
        response = requests.get(url, data=data, **kwargs)
        Http._log_response(logger, response)
        return response

    @staticmethod
    def post(url, data=None, json=None, logger=None, **kwargs):
        if logger is not None:
            Http._log_request(logger, data=data, headers=kwargs.get('headers', None))
        response = requests.post(url, data=data, json=json, **kwargs)
        Http._log_response(logger, response)
        return response

    @staticmethod
    def put(url, data=None, logger=None, **kwargs):
        if logger is not None:
            Http._log_request(logger, data=data, headers=kwargs.get('headers', None))
        response = requests.put(url, data=data, **kwargs)
        Http._log_response(logger, response)
        return response

    @staticmethod
    def delete(url, data=None, logger=None, **kwargs):
        if logger is not None:
            Http._log_request(logger, data=data, headers=kwargs.get('headers', None))
        response = requests.delete(url, data=data, **kwargs)
        Http._log_response(logger, response)
        return response
